@@ -2,7 +2,7 @@ package AnyEvent::Redis;
 
 use strict;
 use 5.008_001;
-our $VERSION = '0.21';
+our $VERSION = '0.22';
 
 use constant DEBUG => $ENV{ANYEVENT_REDIS_DEBUG};
 use AnyEvent;
@@ -273,6 +273,7 @@ sub anyevent_read_type {
                             return $async;
                         }
                     } elsif($hd->{rbuf} =~ /^\*/) { # Nested
+                        my $reader_closure = $reader; # Need to avoid holding circular ref
 
                         $hd->unshift_read(__PACKAGE__, sub {
                                 push @lines, $_[0];
@@ -280,11 +281,14 @@ sub anyevent_read_type {
                                 if(@lines == $size) {
                                     warn "$size nested values" if DEBUG;
                                     $cb->(\@lines);
+                                    undef $reader_closure;
                                 } else {
-                                    $hd->unshift_read($reader);
+                                    $hd->unshift_read($reader_closure);
                                 }
                                 return 1;
                             });
+
+                        undef $reader;
                         return 1;
                     } else {
                         $hd->unshift_read($reader) if $async;
